@@ -22,6 +22,7 @@ import pandas as pd
 
 from .backtesting import Strategy
 from ._plotting import plot_heatmaps as _plot_heatmaps
+from ._stats import compute_stats as _compute_stats
 from ._util import _Array, _as_str
 
 __pdoc__ = {}
@@ -37,7 +38,7 @@ OHLCV_AGG = OrderedDict((
 """Dictionary of rules for aggregating resampled OHLCV data frames,
 e.g.
 
-    df.resample('4H', label='right').agg(OHLCV_AGG)
+    df.resample('4H', label='right').agg(OHLCV_AGG).dropna()
 """
 
 TRADES_AGG = OrderedDict((
@@ -164,6 +165,39 @@ def quantile(series: Sequence, quantile: Union[None, float] = None):
     return np.nanpercentile(series, quantile * 100)
 
 
+def compute_stats(
+        *,
+        stats: pd.Series,
+        data: pd.DataFrame,
+        trades: pd.DataFrame = None,
+        risk_free_rate: float = 0.) -> pd.Series:
+    """
+    (Re-)compute strategy performance metrics.
+
+    `stats` is the statistics series as returned by `backtesting.backtesting.Backtest.run()`.
+    `data` is OHLC data as passed to the `backtesting.backtesting.Backtest`
+    the `stats` were obtained in.
+    `trades` can be a dataframe subset of `stats._trades` (e.g. only long trades).
+    You can also tune `risk_free_rate`, used in calculation of Sharpe and Sortino ratios.
+
+        >>> stats = Backtest(GOOG, MyStrategy).run()
+        >>> only_long_trades = stats._trades[stats._trades.Size > 0]
+        >>> long_stats = compute_stats(stats=stats, trades=only_long_trades,
+        ...                            data=GOOG, risk_free_rate=.02)
+    """
+    equity = stats._equity_curve.Equity
+    if trades is None:
+        trades = stats._trades
+    else:
+        # XXX: Is this buggy?
+        equity = equity.copy()
+        equity[:] = stats._equity_curve.Equity.iloc[0]
+        for t in trades.itertuples(index=False):
+            equity.iloc[t.EntryBar:] += t.PnL
+    return _compute_stats(trades=trades, equity=equity, ohlc_data=data,
+                          risk_free_rate=risk_free_rate, strategy_instance=stats._strategy)
+
+
 def resample_apply(rule: str,
                    func: Optional[Callable[..., Sequence]],
                    series: Union[pd.Series, pd.DataFrame, _Array],
@@ -282,9 +316,9 @@ http://pandas.pydata.org/pandas-docs/stable/timeseries.html#offset-aliases
             elif result.ndim == 2:
                 result = pd.DataFrame(result.T)
         # Resample back to data index
-        if not result.index.is_all_dates:
+        if not isinstance(result.index, pd.DatetimeIndex):
             result.index = resampled.index
-        result = result.reindex(index=series.index | resampled.index,
+        result = result.reindex(index=series.index.union(resampled.index),
                                 method='ffill').reindex(series.index)
         return result
 
@@ -317,7 +351,7 @@ def random_ohlc_data(example_data: pd.DataFrame, *,
     def shuffle(x):
         return x.sample(frac=frac, replace=frac > 1, random_state=random_state)
 
-    if len(example_data.columns & {'Open', 'High', 'Low', 'Close'}) != 4:
+    if len(example_data.columns.intersection({'Open', 'High', 'Low', 'Close'})) != 4:
         raise ValueError("`data` must be a pandas.DataFrame with columns "
                          "'Open', 'High', 'Low', 'Close'")
     while True:
@@ -356,8 +390,6 @@ class SignalStrategy(Strategy):
     __entry_signal = (0,)
     __exit_signal = (False,)
 
-    __pdoc__['SignalStrategy.__init__'] = False
-
     def set_signal(self, entry_size: Sequence[float],
                    exit_portion: Sequence[float] = None,
                    *,
@@ -376,12 +408,12 @@ class SignalStrategy(Strategy):
         If `plot` is `True`, the signal entry/exit indicators are plotted when
         `backtesting.backtesting.Backtest.plot` is called.
         """
-        self.__entry_signal = self.I(
+        self.__entry_signal = self.I(  # type: ignore
             lambda: pd.Series(entry_size, dtype=float).replace(0, np.nan),
             name='entry size', plot=plot, overlay=False, scatter=True, color='black')
 
         if exit_portion is not None:
-            self.__exit_signal = self.I(
+            self.__exit_signal = self.I(  # type: ignore
                 lambda: pd.Series(exit_portion, dtype=float).replace(0, np.nan),
                 name='exit portion', plot=plot, overlay=False, scatter=True, color='black')
 
@@ -420,8 +452,6 @@ class TrailingStrategy(Strategy):
     __n_atr = 6.
     __atr = None
 
-    __pdoc__['SignalStrategy.__init__'] = False
-
     def init(self):
         super().init()
         self.set_atr_periods()
@@ -445,13 +475,21 @@ class TrailingStrategy(Strategy):
 
     def next(self):
         super().next()
+        # Can't use index=-1 because self.__atr is not an Indicator type
+        index = len(self.data)-1
         for trade in self.trades:
             if trade.is_long:
                 trade.sl = max(trade.sl or -np.inf,
-                               self.data.Close[-1] - self.__atr[-1] * self.__n_atr)
+                               self.data.Close[index] - self.__atr[index] * self.__n_atr)
             else:
                 trade.sl = min(trade.sl or np.inf,
-                               self.data.Close[-1] + self.__atr[-1] * self.__n_atr)
+                               self.data.Close[index] + self.__atr[index] * self.__n_atr)
+
+
+# Prevent pdoc3 documenting __init__ signature of Strategy subclasses
+for cls in list(globals().values()):
+    if isinstance(cls, type) and issubclass(cls, Strategy):
+        __pdoc__[f'{cls.__name__}.__init__'] = False
 
 
 # NOTE: Don't put anything below this __all__ list
