@@ -4,6 +4,7 @@ import sys
 import time
 import unittest
 import warnings
+from concurrent.futures.process import ProcessPoolExecutor
 from contextlib import contextmanager
 from glob import glob
 from runpy import run_path
@@ -13,11 +14,14 @@ from unittest.mock import patch
 
 import numpy as np
 import pandas as pd
+from pandas.testing import assert_frame_equal
 
 from backtesting import Backtest, Strategy
+from backtesting._stats import compute_drawdown_duration_peaks
 from backtesting.lib import (
     OHLCV_AGG,
     barssince,
+    compute_stats,
     cross,
     crossover,
     quantile,
@@ -242,7 +246,7 @@ class TestBacktest(TestCase):
 
     def test_compute_drawdown(self):
         dd = pd.Series([0, 1, 7, 0, 4, 0, 0])
-        durations, peaks = Backtest._compute_drawdown_duration_peaks(dd)
+        durations, peaks = compute_drawdown_duration_peaks(dd)
         np.testing.assert_array_equal(durations, pd.Series([3, 2], index=[3, 5]).reindex(dd.index))
         np.testing.assert_array_equal(peaks, pd.Series([7, 4], index=[3, 5]).reindex(dd.index))
 
@@ -642,6 +646,14 @@ class TestPlot(TestCase):
                 with self.subTest(param=p[0]):
                     bt.plot(**dict([p]), filename=f, open_browser=False)
 
+    def test_hide_legend(self):
+        bt = Backtest(GOOG.iloc[:100], SmaCross)
+        bt.run()
+        with _tempfile() as f:
+            bt.plot(filename=f, show_legend=False)
+            # Give browser time to open before tempfile is removed
+            time.sleep(5)
+
     def test_resolutions(self):
         with _tempfile() as f:
             for rule in 'LSTHDWM':
@@ -789,8 +801,9 @@ class TestLib(TestCase):
         res = resample_apply('D', SMA, EURUSD.Close, 10)
         self.assertEqual(res.name, 'C[D]')
         self.assertEqual(res.count() / res.size, .9634)
-        self.assertEqual(res.iloc[-48:].unique().tolist(),
-                         [1.2426429999999997, 1.2423809999999995, 1.2422749999999998])
+        np.testing.assert_almost_equal(res.iloc[-48:].unique().tolist(),
+                                       [1.242643, 1.242381, 1.242275],
+                                       decimal=6)
 
         def resets_index(*args):
             return pd.Series(SMA(*args).values)
@@ -826,6 +839,18 @@ class TestLib(TestCase):
         self.assertEqual(new_data.shape, GOOG.shape)
         self.assertEqual(list(new_data.columns), list(GOOG.columns))
 
+    def test_compute_stats(self):
+        stats = Backtest(GOOG, SmaCross).run()
+        only_long_trades = stats._trades[stats._trades.Size > 0]
+        long_stats = compute_stats(stats=stats, trades=only_long_trades,
+                                   data=GOOG, risk_free_rate=.02)
+        self.assertNotEqual(list(stats._equity_curve.Equity),
+                            list(long_stats._equity_curve.Equity))
+        self.assertNotEqual(stats['Sharpe Ratio'], long_stats['Sharpe Ratio'])
+        self.assertEqual(long_stats['# Trades'], len(only_long_trades))
+        self.assertEqual(stats._strategy, long_stats._strategy)
+        assert_frame_equal(long_stats._trades, only_long_trades)
+
     def test_SignalStrategy(self):
         class S(SignalStrategy):
             def init(self):
@@ -834,7 +859,7 @@ class TestLib(TestCase):
                                 self.data.Close < sma)
 
         stats = Backtest(GOOG, S).run()
-        self.assertEqual(stats['# Trades'], 1182)
+        self.assertIn(stats['# Trades'], (1181, 1182))  # varies on different archs?
 
     def test_TrailingStrategy(self):
         class S(TrailingStrategy):
@@ -850,7 +875,7 @@ class TestLib(TestCase):
                     self.buy()
 
         stats = Backtest(GOOG, S).run()
-        self.assertEqual(stats['# Trades'], 51)
+        self.assertEqual(stats['# Trades'], 57)
 
 
 class TestUtil(TestCase):
@@ -890,6 +915,13 @@ class TestUtil(TestCase):
 
         Backtest(GOOG.iloc[:20], S).run()
 
+    def test_indicators_picklable(self):
+        bt = Backtest(SHORT_DATA, SmaCross)
+        with ProcessPoolExecutor() as executor:
+            stats = executor.submit(Backtest.run, bt).result()
+        assert stats._strategy._indicators[0]._opts, '._opts and .name were not unpickled'
+        bt.plot(results=stats, resample='2d', open_browser=False)
+
 
 class TestDocs(TestCase):
     DOCS_DIR = os.path.join(os.path.dirname(__file__), '..', '..', 'doc')
@@ -900,7 +932,8 @@ class TestDocs(TestCase):
         self.assertGreaterEqual(len(examples), 4)
         with chdir(gettempdir()):
             for file in examples:
-                run_path(file)
+                with self.subTest(example=os.path.basename(file)):
+                    run_path(file)
 
     def test_backtest_run_docstring_contains_stats_keys(self):
         stats = Backtest(SHORT_DATA, SmaCross).run()
